// **************************************
// ModuleList class definitions
// **************************************

#if !defined(MODULELIST_H)
#define MODULELIST_H

// **************************
// Headers
// **************************

#include "module.h"
#include "stringarray.h"

// **************************
// ModuleList class
// **************************

class ModuleList {

public:
  /// Constructor.
  /**
  * Create array with <i>l</i> elements.
  */
  ModuleList(const unsigned int l=0);

  /// Copy constructor.
  /**
  * Copy all array elements.
  */
  ModuleList(const ModuleList& array);

  /// Destructor.
  /**
  * Free memory.
  */
  virtual ~ModuleList();

  /// Assignment operator.
  /**
  * Copy all array elements.
  */
  ModuleList& operator=(const ModuleList& array);

  /// Delete all values.
  void Clear(void);

  /// Alternate assignment operator.
  /**
  * Copy all array elements.
  */
  void Set(const ModuleList& array);

  /// Access operator.
  /**
  * Return entry or dummy if <i>index</i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  const Module& operator()(const unsigned int index) const;

  /// Access operator (as reference).
  /**
  * Return entry or dummy if <i><b>index</b></i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  Module& operator()(const unsigned int index);

  /// Append element.
  void Append(const Module& value);

  /// Append all elements from other array.
  void Append(const ModuleList& array);

  /// Insert value at index.
  /**
  * If <i>index == size</i> the value will be appended.
  *
  * Return <b>true</b> if <i>index &gt; size</i> else <b>false</b>.
  *
  * See also: Append(const Module&)
  */
  const bool Insert(const unsigned int index, const Module& value);

  /// Insert array at index.
  /**
  * If <i>index == size</i> the array will be appended.
  *
  * Return <b>true</b> if <i>index &gt; size</i> else <b>false</b>.
  *
  * See also: Append(const ModuleList&)
  */
  const bool Insert(const unsigned int index, const ModuleList& array);

  /// Delete <i>width</i> elements from <i>index</i>.
  /**
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool Delete(const unsigned int index, unsigned int width=1);

  /// Copy array elements to <i>index</i>.
  /**
  * If <i>size &lt; index+array.size</i>, the array will be extended.
  *
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool Set(const unsigned int index, const ModuleList& array);

  /// Extend array to new size.
  void Extend(const unsigned int newsize);

  /// Truncate array to new size.
  void Trunc(const unsigned int newsize);

  /// Set new array size.
  /**
  * See also: Extend(const unsigned int), Trunc(const unsigned int)
  */
  void SetSize(const unsigned int newsize);

  /// Return size of array.
  const unsigned int GetSize(void) const;

  /// Get comments.
  const StringArray& Comments(void) const;

  /// Get comments (as reference).
  StringArray& Comments(void);

  /// Return current element.
  const unsigned int GetCurrent(void) const;

  /// Set current element.
  /**
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool SetCurrent(const unsigned int index);

  /// Return current element.
  /**
  * Return entry or dummy if <i>current</i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  const Module& Current(void) const;

  /// Return current element (as reference).
  /**
  * Return entry or dummy if <i><b>current</b></i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  Module& Current(void);

  /// Set current element at last position.
  /**
  * Return <b>false</b> every time. It may be necessary
  * to return <b>true</b> if <i>size==0</i>.
  */
  const bool SetLast(void);

  /// Delete current element.
  /**
  * Return <b>false</b> every time. It may be necessary
  * to return <b>true</b> if <i>size==0</i>.
  */
  const bool DeleteCurrent(void);

  /// Check if everything has correct syntax.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(void);

private:
  Module* values;        ///< Array elements.
  unsigned int size;        ///< Size of array.
  unsigned int current;     ///< Current element.

  StringArray comments;  ///< Comments.

  /// Dummy element.
  /**
  * This element is necessary for returning a reference
  * if the array access is out of boundaries.
  *
  * See also: operator()(const unsigned int)
  */
  static Module dummy;

};

#endif // !defined(MODULELIST_H)

